static struct comnd {        // общее для всех программ описание набора команд
   string      mnemo;
   int         cod, length, ops;
   string      pattern;      // регулярное выражение синтаксиса
   const void* func;
} comnds[] = {
   { "halt"  , 0x00,  1, 0,   // halt
     R"(^\s*(halt)\s*(#.*)?$)"     },
   { "nop"   , 0x10,  1, 0,   // nop
     R"(^\s*(nop)\s*(#.*)?$)"      },
   { "rrmovq", 0x20,  2, 2,   // rrmovq    rA, rB
     R"(^\s*(rrmovq)\s+%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*,\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
   { "irmovq", 0x30, 10, 2,   // irmovq    D, rB
     R"(^\s*(irmovq)\s+(\w+|\$[\+|\-]?[0-9]+|\$0x[0-9|a-f]+)\s*,\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
   { "rmmovq", 0x40, 10, 2,   // rmmovq    rA, D(rB)  
     R"(^\s*(rmmovq)\s+%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*,\s*([\+|\-]?[0-9]+|0x[0-9|a-f]+)*\s*\(\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*\)\s*(#.*)?$)" },
   { "mrmovq", 0x50, 10, 2,   // mrmovq    D(rB), rA 
     R"(^\s*(mrmovq)\s+([\+|\-]?[0-9]+|0x[0-9|a-f]+)*\s*\(\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*\)\s*,\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
   { "addq"  , 0x60,  2, 2,   // addq      rA, rB 
     R"(^\s*(addq)\s+%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*,\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
   { "subq"  , 0x61,  2, 2,   // subq      rA, rB
     R"(^\s*(subq)\s+%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*,\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
   { "andq"  , 0x62,  2, 2,   // andq      rA, rB 
     R"(^\s*(andq)\s+%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*,\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
   { "xorq"  , 0x63,  2, 2,   // xorq      rA, rB
     R"(^\s*(xorq)\s+%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*,\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
//-----------------------------------------------------------------------------------------------
   { "cmovle", 0x21,  2, 2,   // cmovle    rA, rB
     R"(^\s*(cmovle)\s+%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*,\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
   { "cmovl" , 0x22,  2, 2,   // cmovl     rA, rB
     R"(^\s*(cmovl)\s+%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*,\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
   { "cmove" , 0x23,  2, 2,   // cmove     rA, rB
     R"(^\s*(cmove)\s+%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*,\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
   { "cmovne", 0x24,  2, 2,   // cmovne    rA, rB 
     R"(^\s*(cmovne)\s+%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*,\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
   { "cmovge", 0x25,  2, 2,   // cmovge    rA, rB 
     R"(^\s*(cmovge)\s+%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*,\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
   { "cmovg" , 0x26,  2, 2,   // cmovg     rA, rB
     R"(^\s*(cmovg)\s+%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*,\s*%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
//-----------------------------------------------------------------------------------------------
   { "jmp"   , 0x70,  9, 1,   // jmp       A
     R"(^\s*(jmp)\s+(\w+)\s*(#.*)?$)" },
   { "jle"   , 0x71,  9, 1,   // jle       A
     R"(^\s*(jle)\s+(\w+)\s*(#.*)?$)" },
   { "jl"    , 0x72,  9, 1,   // jl        A  
     R"(^\s*(jl)\s+(\w+)\s*(#.*)?$)" },
   { "je"    , 0x73,  9, 1,   // je        A 
     R"(^\s*(je)\s+(\w+)\s*(#.*)?$)" },
   { "jne"   , 0x74,  9, 1,   // jne       A
     R"(^\s*(jne)\s+(\w+)\s*(#.*)?$)" },
   { "jge"   , 0x75,  9, 1,   // jge       A
     R"(^\s*(jge)\s+(\w+)\s*(#.*)?$)" },
   { "jg"    , 0x76,  9, 1,   // jg"       A
     R"(^\s*(jg)\s+(\w+)\s*(#.*)?$)" },
   { "call"  , 0x80,  9, 1,   // call      A
     R"(^\s*(call)\s+(\w+)\s*(#.*)?$)" },
   { "ret"   , 0x90,  1, 0,   // ret
     R"(^\s*(ret)\s*(#.*)?$)" },

   { "pushq" , 0xA0,  2, 1,   // pushl     ra
     R"(^\s*(pushq)\s+%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
   { "popq"  , 0xB0,  2, 1,   // popl      ra 
     R"(^\s*(popq)\s+%r([a|c|d|b]x|sp|bp|si|di|[8-9]|1[0-4])\s*(#.*)?$)" },
};

const int ncomnd = sizeof( comnds ) / sizeof( struct comnd ); 

