class command {    // базовый класс command с перегруженной операцией operator()
public:	
   virtual void operator() ( Y86*, struct comnd* ) = 0;
};

class noop : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
      if( "halt" == c->mnemo ) 
         p->state = p->HALT;
      else
         p->IP += c->length;
   }
};

class rrmov : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
	  uint8_t op = p->ram[ p->IP + 1 ];
	  uint8_t rB = op & 0xf, rA = ( op >> 4 ) & 0xf;
      if( ( "rrmov" == c->mnemo.substr( 0, 5 ) ) ||
          ( "cmovle" == c->mnemo && ( p->ZF || p-> SF ) ) ||
          ( "cmovl" == c->mnemo && ( p->SF ) ) ||
          ( "cmove" == c->mnemo && ( p->ZF ) ) || 
          ( "cmovne" == c->mnemo && ( !p->ZF ) ) ||
          ( "cmovge" == c->mnemo && ( p->ZF || !p-> SF ) ) ||
          ( "cmovg"  == c->mnemo && ( !p-> SF ) ) ) p->R[ rB ] = p->R[ rA ];  
      p->IP += c->length;
   }
};

class irmov : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
	  uint8_t rB = p->ram[ p->IP + 1 ] & 0xf;
	  uint64_t op = p->value_at( p->IP + 2 );
      p->R[	rB ] = op;   
      p->IP += c->length;
   }
};

class rmmov : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
	  uint8_t op = p->ram[ p->IP + 1 ];
   	  uint8_t rB = op & 0xf, rA = ( op >> 4 ) & 0xf;
	  uint64_t sh = p->value_at( p->IP + 2 );
	  addr_t addr = p->R[ rB ] + sh;
      p->save64reg( rA, addr );	
      p->IP += c->length;
   }
};

class mrmov : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
	  uint8_t op = p->ram[ p->IP + 1 ];
   	  uint8_t rB = op & 0xf, rA = ( op >> 4 ) & 0xf;
	  uint64_t sh = p->value_at( p->IP + 2 ); 
	  addr_t addr = p->R[ rB ] + sh;
      p->R[ rA ] = p->value_at( addr );
      p->IP += c->length;
   }
};

class arithm : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
	  uint8_t op = p->ram[ p->IP + 1 ];
   	  uint8_t rA = ( op >> 4 ) & 0xf, rB = op & 0xf; // [rA] op [rB] => [rB]
      int64_t v1 = p->R[ rA ], v2 = p->R[ rB ], res;
//      int64_t res; // = v2;
      bool ok = true; 
      if( "add" == c->mnemo.substr( 0, 3 ) ) { 
         res = v2 + v1;
         p->SF = res < 0;
		 p->OF = ( res < v1 ) && ( res < v2 );
      }
      else if( "sub" == c->mnemo.substr( 0, 3 ) ) { 
		 res = v2 - v1;
         p->SF = res < 0;		 
		 p->OF = res > v2;
      }
      else if( "and" == c->mnemo.substr( 0, 3 ) ) {
		 res = v2 & v1;
		 p->SF = p->OF = false; 
	  }
      else if( "xor" == c->mnemo.substr( 0, 3 ) ) {
		 res = v2 ^ v1;
		 p->SF = p->OF = false; 
	  }		 
      else {
		 ok = false;
         cerr << "недопустимый код арифметической операции: "
              << c->mnemo << endl;   
	  }
	  if( ok ) {
         p->R[ rB ] = res; 
         p->ZF = 0 == res;
      }
      p->IP += c->length;
   }
};

class jmp : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
	  if( ( "jmp" == c->mnemo ) ||
	      ( "jle" == c->mnemo && ( p->ZF || p-> SF ) ) ||	  	  
          ( "jl"  == c->mnemo && ( p->SF ) ) ||
          ( "je"  == c->mnemo && ( p->ZF ) ) ||
          ( "jne" == c->mnemo && ( !p->ZF ) ) ||
          ( "jge" == c->mnemo && ( p->ZF || !p-> SF ) ) ||
          ( "jg"  == c->mnemo && ( !p-> SF ) ) ) {
	     addr_t addr = p->value_at( p->IP + 1 );  
	     p->IP = addr;
	  }
      else 
         p->IP += c->length;
   }
};

const int SP = 4; // номер регистра указателя стека %esp

class call : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
      p->R[ SP ] -= sizeof( uint64_t );
      p->save64( p->IP + c->length, p->R[ SP ] );
      addr_t addr = p->value_at( p->IP + 1 ); 
      p->IP = addr; 
   }
};

class ret : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
      addr_t addr = p->value_at( p->R[ SP ] );
      p->R[ SP ] += sizeof( uint64_t ); 
      p->IP = addr; 
   }
};

class push_pop : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
	  uint8_t rA = p->ram[ p->IP + 1 ] >> 4 & 0xf; // номер регистра 
	  if( "push" == c->mnemo.substr( 0, 4 ) ) {
         p->R[ SP ] -= sizeof( uint64_t );
         p->save64reg( rA, p->R[ SP ] );
	  }
	  else if ( "pop" == c->mnemo.substr( 0, 3 ) ) {
		 p->load64reg( rA, p->R[ SP ] ); 		 
         p->R[ SP ] += sizeof( uint64_t ); 
	  }
      else
         cerr << "недопустимый код операции: " << c->mnemo << endl;   
      p->IP += c->length;
   }
};

static bool init_table( void ) {    // функция стартовой инициализации
   map< string, command* > op_table = {
      { "halt"  , new noop     },   // halt 
      { "nop"   , new noop     },   // nop
      { "rrmovq", new rrmov    },   // rrmovq    rA -> rB
      { "irmovq", new irmov    },   // irmovq     V -> rB
      { "rmmovq", new rmmov    },   // rmmovq    rA -> D(rB)  
      { "mrmovq", new mrmov    },   // mrmovq D(rB) -> rA 
      { "addq"  , new arithm   },   // addq      rA + rB -> rB 
      { "subq"  , new arithm   },   // subq      rA - rB -> rB
      { "andq"  , new arithm   },   // andq      rA & rB -> rB 
      { "xorq"  , new arithm   },   // xorq      rA ^ rB -> rB
      { "cmovle", new rrmov    },   // cmovle    rA, rB
      { "cmovl" , new rrmov    },   // cmovl     rA, rB
      { "cmove" , new rrmov    },   // cmove     rA, rB
      { "cmovne", new rrmov    },   // cmovne    rA, rB 
      { "cmovge", new rrmov    },   // cmovge    rA, rB  
      { "cmovg" , new rrmov    },   // cmovg     rA, rB
      { "jmp"   , new jmp      },   // jmp       A
      { "jle"   , new jmp      },   // jle       A
      { "jl"    , new jmp      },   // jl        A  
      { "je"    , new jmp      },   // je        A 
      { "jne"   , new jmp      },   // jne       A
      { "jge"   , new jmp      },   // jge       A
      { "jg"    , new jmp      },   // jg"       A
      { "call"  , new call     },   // call      A
      { "ret"   , new ret      },   // ret
      { "pushq" , new push_pop },   // pushl     rA
      { "popq"  , new push_pop }    // popl      rA
   };
   for( int i = 0; i < ncomnd; i++ ) 
      try {
         comnds[ i ].func = (const void*)op_table.at( comnds[ i ].mnemo );
      }
      catch( out_of_range const& e ) {
         cerr << "несоответствие таблиц: " << comnds[ i ].mnemo << endl; 
		 return false;
      }  
   if( ncomnd != op_table.size() ) {
      cerr << "несоответствие размеров таблиц" << endl; 
	  return false;
   }
   return true;
}
