//-------------------------- разбор операндов -------------------------------
typedef string (*op_func)( smatch&, map< string, addr_t >& );
 
const map< string, char > registers = {
   { "ax", '0' }, { "cx", '1' }, { "dx", '2' }, { "bx", '3' },
   { "sp", '4' }, { "bp", '5' }, { "si", '6' }, { "di", '7' },
   { "8" , '8' }, { "9" , '9' }, { "10", 'a' }, { "11", 'b' },
   { "12", 'c' }, { "13", 'd' }, { "14", 'e' } };

string oper0( smatch& m, map< string, addr_t >& lbl ) {
   return "";
} 

string irmov( smatch& m, map< string, addr_t >& lbl ) {
   char op[ 3 ] = "f0";
   op[ 1 ] = registers.at( m[ 3 ].str() );
   string ret( op );
   if( '$' == m[ 2 ].str()[ 0 ] )                      // числовой операнд
      ret += load_arg( get_number( m[ 2 ].str().substr( 1 ) ) );      
   else 
      try {                                            // адрес метки
         ret += load_arg( lbl.at( m[ 2 ].str() ) );
      }
      catch( out_of_range const& e ) {
         cout << "неопределённое имя метки: " << m[ 2 ].str() << endl;
         exit( 5 );
      }
   return ret;
}

string rmmov( smatch& m, map< string, addr_t >& lbl ) {
   char op[ 3 ] = "00";
   op[ 0 ] = registers.at( m[ 2 ].str() );
   op[ 1 ] = registers.at( m[ 4 ].str() );
   return string( op ) + load_arg( get_number( m[ 3 ].str() ) );
}

string mrmov( smatch& m, map< string, addr_t >& lbl ) {
   char op[ 3 ] = "00";
   op[ 0 ] = registers.at( m[ 4 ].str() );
   op[ 1 ] = registers.at( m[ 3 ].str() );
   return string( op ) + load_arg( get_number( m[ 2 ].str() ) );
}

string arithm( smatch& m, map< string, addr_t >& lbl ) {
   char op[ 3 ] = "00";
   op[ 0 ] = registers.at( m[ 2 ].str() );
   op[ 1 ] = registers.at( m[ 3 ].str() );
   return string( op );
}

string jmp( smatch& m, map< string, addr_t >& lbl ) {
   addr_t addr;
   try {
      addr = lbl.at( m[ 2 ].str() );
   }
   catch( exception const& e ) {
      cout << "неопределённое имя метки: " << m[ 2 ].str() << endl;
      exit( 5 );
   }
   return load_arg( addr );
}

string push_pop( smatch& m, map< string, addr_t >& lbl ) {
   char op[ 3 ] = "0f";
   op[ 0 ] = registers.at( m[ 2 ].str() );
   return string( op );
}

static int init = []( void ) -> bool {                 // анонимная функция стартовой инициализации
   const map< string, op_func > op_table = {
      { "halt"  , oper0    },        // halt 
      { "nop"   , oper0    },        // nop
      { "rrmovq", arithm   },        // rrmovq    rA, rB
      { "irmovq", irmov    },        // irmovq     V, rB
      { "rmmovq", rmmov    },        // rmmovq    rA, D(rB)  
      { "mrmovq", mrmov    },        // mrmovq D(rB), rA 
      { "addq"  , arithm   },        // addq      rA, rB 
      { "subq"  , arithm   },        // subq      rA, rB
      { "andq"  , arithm   },        // andq      rA, rB 
      { "xorq"  , arithm   },        // xorq      rA, rB
      { "cmovle", arithm   },        // cmovle    rA, rB
      { "cmovl" , arithm   },        // cmovl     rA, rB
      { "cmove" , arithm   },        // cmove     rA, rB
      { "cmovne", arithm   },        // cmovne    rA, rB
      { "cmovge", arithm   },        // cmovge    rA, rB
      { "cmovg" , arithm   },        // cmovg     rA, rB
      { "jmp"   , jmp      },        // jmp       A
      { "jle"   , jmp      },        // jle       A
      { "jl"    , jmp      },        // jl        A  
      { "je"    , jmp      },        // je        A 
      { "jne"   , jmp      },        // jne       A
      { "jge"   , jmp      },        // jge       A
      { "jg"    , jmp      },        // jg"       A  
      { "call"  , jmp      },        // call      A 
      { "ret"   , oper0    },        // ret
      { "pushq" , push_pop },        // pushq     rA
      { "popq"  , push_pop }         // popq      rA 
   };
   for( int i
    = 0; i < ncomnd; i++ ) 
      try {
         comnds[ i ].func = (const void*)op_table.at( comnds[ i ].mnemo );
      }
      catch( out_of_range const& e ) {
         cerr << "несоответствие таблиц: " << comnds[ i ].mnemo << endl; 
		 exit( 6 );
      }  
   if( ncomnd != op_table.size() ) {
      cerr << "несоответствие таблиц" << endl; 
	  exit( 6 );	   
   }
   return ncomnd; 
} (); 
