#include "common.h"

#include "command.cc"

int64_t get_number( const string& snumb ) {
   if( snumb.empty() ) return 0;
   if( '-' == snumb[ 0 ] )
      return -1 * get_number( snumb.substr( 1 ) );
   else if( '0' == snumb[ 0 ] && 'x' == snumb[ 1 ] )
      return stoull( snumb.substr( 2 ), NULL, 16 );
   else
      return stoull( snumb );
}

string load_arg( uint64_t arg ) {                           // представление целого числа в памяти
   char s[ sizeof( arg ) * 2 + 1 ];
   sprintf( s, "%016lx", arg );
   string ret( s );
   if( little_endian ) {                                    // изменение порядка байт
      auto b = ret.begin(), e = ret.end() - 1;
      while( b < e ) {
         swap( *b, *( e - 1 ) );
         swap( *( b + 1 ), *e );
         b += 2; e -= 2;
      }
   }
   return ret;
}

#include "operandos.cc"

class text : protected vector<string> {
   const int CMDLEN=10;                                     // max. длина записи команды 
   addr_t m_addr;                                           // текущий адрес при компиляции
   bool m_resolve;                                          // разрешать имена (2-й проход)
   map< string, addr_t > m_ltab;                            // таблица адресов меток                                                        
   string lables( const string& );                          // сбор адресов меток (1-й проход)
   string directive( const string& );                       // обработка директивы ассемблера
   string decode( const string& );                          // компиляция строки команды
public:
   friend istream& operator >>( istream& inp, text& t ) {   // загрузка исходного кода
      bool file = strstr( typeid( inp ).name(), "ifstream" );
      if( !file )
         cout << "input text (for final - ^D):" << endl;
      t.clear();
      string line;
      do {
         getline( inp, line );
         t.push_back( line );
      } while( !inp.eof() );
      return inp;
   }
   vector<string>& compile( void );                         // компиляция исходного кода
};

string text::lables( const string& cmd ) {                  // сбор адресов меток из строки
   string::size_type found = cmd.find_first_not_of( " \t" );
   if( string::npos == found || '#' == cmd[ found ] )       // комментарий или пустая строка 
      return cmd;
   string line( cmd );
   static regex r = pattern( R"(^\s*(\w+\s*:)+\s*(.*)\s*$)" );
   while( true ) {
      smatch m;
      regex_match( line, m, r );
      if( 0 == m.size() ) break;
      string labl = m[ 1 ].str().substr( 0, m.length( 1 ) - 1 );
      line = m[ 2 ].str();
      m_ltab.insert( pair< string, unsigned >( labl, m_addr ) );
   }
   if( 0 == line.length() ) return line;
   found = line.find_first_not_of( " \t" );
   if( found != string::npos && '.' == line[ found ] )      // 1-й проход 
      directive( line );
   else 
      decode( line );
   return line;
}

struct dir {                                                // рег. выражения директив ассемблера
   string mnemo;
   int    length;
   string pattern;
} dirs[] = {
   { ".pos"  , 0, R"(^\s*(.pos)\s+([0-9]{1,10}|0x[0-9|a-f]{1,8})\s*(#.*)?$)" },
   { ".align", 0, R"(^\s*(.align)\s+(1|2|4|8|16|32|64)\s*$)" }, 
//   { ".quad" , 8, R"(^\s*(.quad)\s+([\+|\-]?[0-9]{1,10}|0x[0-9|a-f]{1,8})\s*(#.*)?$)" }   
   { ".quad" , 8, R"(^\s*(.quad)\s+([\+|\-]?[0-9]{1,20}|0x[0-9|a-f]{1,16})\s*(#.*)?$)" }   
           };

string text::directive( const string& cmd ) {               // обработка директив ассемблера
   smatch m;
   struct dir curr;
   for( auto &x : dirs ) 
      if( regex_match( cmd, m, pattern( x.pattern ) ) ) {
         curr = x;
         break;
      }
   if( 0 == m.size() ) {
      cerr << "недопустимая деректива ассемблера: " << cmd << endl;
      exit( 4 );
   }
   addr_t arg = get_number( m[ 2 ].str() );
   ostringstream oret;
   oret << "";
   if( ".pos" == m[ 1 ].str() )
      m_addr = arg;
   else if( ".align" == m[ 1 ].str() )
      m_addr = ( m_addr / arg  + 1 ) * arg;
   else if( ".quad" == m[ 1 ].str() ) {
      m_addr += curr.length;
      oret << setw( curr.length ) << load_arg( arg );
   }
   return oret.str();
}

string text::decode( const string& cmd ) {                  // компиляция строки команды
   if( !cmd.length() ) return cmd;                          // пустая строка с меткой
   smatch m;
   struct comnd curr;
   for( auto &x : comnds ) 
      if( regex_match( cmd, m, pattern( x.pattern ) ) ) {
         curr = x;
         break;
      }
   if( 0 == m.size() ) {
      cerr << "недопустимый код команды: " << cmd << endl;
      exit( 4 );
   }
   m_addr += curr.length;
   ostringstream oret;
   if( m_resolve ) {                                        // генерировать только на 2-м проходе
      oret << hex << setw( 2 ) << setfill( '0' )
           << curr.cod;                                     // код команды
      oret << ( (op_func)curr.func )( m, m_ltab );          // функция обработки операндов
   }
   return oret.str();
}

vector<string>& text::compile( void ) {                     // компилированный исходный код      
   static vector<string> tmp, res;
   m_addr = 0; m_resolve = false;
   for( auto &x : (*this) )                                 // 1-й проход: сбор адресов меток
      tmp.push_back( lables( x ) );
   if( debug_level > 0 ) { 
      cerr << "----------------------------------------------" << endl;   
      for( auto &x : m_ltab )
         cerr << x.first << "\t:\t0x" << hex << setw( ADRLEN * 2 ) 
              << setfill( '0' ) << x.second << endl;
      cerr << "----------------------------------------------" << endl;   
   }
   m_addr = 0; m_resolve = true;
   for( unsigned i = 0; i < size(); i++ ) {                 // 2-й проход: генерация кода
      string x = tmp[ i ];
      ostringstream oret;
      oret << "  ";
      string::size_type found = (*this)[ i ].find_first_not_of( " \t" );
      if( string::npos == found || '#' == x[ found ] )      // комментарий или пустая строка 
         oret << setw( ADRLEN * 2 + CMDLEN * 2 + 4 ) << setfill( ' ' ) << "";
      else {
         string::size_type found = x.find_first_not_of( " \t" );
         if( found != string::npos && '.' == x[ found ] ) { // директива ассемблера
            uint32_t start = m_addr;
            string reply = directive( x );
            oret << "0x" << hex << setw( ADRLEN * 2 ) << setfill( '0' )
                 << ( reply.empty() ? m_addr : start ) << ": "
                 << setw( CMDLEN * 2 ) << setfill( ' ' ) << left << reply;
         }
         else {                                             // код команды
            oret << "0x" << hex << setw( ADRLEN * 2 ) << setfill( '0' ) << m_addr << ": ";
            string reply = decode( x ); 
            oret << setw( CMDLEN * 2 ) << setfill( ' ' ) << left << reply;
         }
      }
      oret << " | " << (*this)[ i ];
      res.push_back( oret.str() );
   }
   return res;
}

int main( int argc, char *argv[] ) {
   string oname, 
          help = "допустимый формат команды: " + string( argv[ 0 ] ) + 
                 " [-v][-g|l][-V] [file.ys] [-o file.yo]";
   char sopt[] = "o:lgVv", c;
   while( -1 != ( c = getopt( argc, argv, sopt ) ) )
      switch( c ) {
         case 'o':
            oname = optarg;
            break;
         case 'l':
            little_endian = true;
            break;
         case 'g':
            little_endian = false;
            break;
         case 'v':
            debug_level++;
            break;
         case 'V':
            cerr << "Y86 compiler 64-bit, version " << version << endl;
            return 0;
         default :
            cerr << help << endl;
            return 1;
      }
   if( argc - optind > 1 ) {
      cerr << help << endl;
      return 1;
   }
   if( debug_level > 0 )                                    // отладка
      cerr << "размерность таблицы команд: " << ncomnd << endl;
   text code;
   if( 1 == argc - optind ) {
      string iname( argv[ optind ] );
      if( string::npos == iname.find( '.' ) )
         iname += ".ys";
      ifstream fin;
      fin.open( iname );
      if( !fin ) {
         cerr << "ошибочное имя входного файла!" << endl;
         return 1;
      }
      fin >> code;
      fin.close();
   }
   else
      cin >> code;
   vector<string> bin = code.compile();
   if( oname != "" ) {
      if( string::npos == oname.find( '.' ) )
         oname += ".yo";
      ofstream fou;
      fou.open( oname );
      if( !fou ) {
         cerr << "ошибочное имя выходного файла!" << endl;
         return 1;
      }
      fou << bin;
      cerr << "в файл " << oname << " записано " << bin.size() << " строк" << endl; 
      fou.close();
   }
   else
      cout << bin;
}
