#include "processor.h"

int main( int argc, char *argv[] ) {
   char sopt[] = "lgv", c;
   string help = "допустимый формат команды: " + string( argv[ 0 ] ) + " [-v][-g|l] file.yo";
   while( -1 != ( c = getopt( argc, argv, sopt ) ) )
      switch( c ) {
         case 'l':
            little_endian = true;
            break;
         case 'g':
            little_endian = false;
            break;
         case 'v':
            debug_level++;
            break;
         default :
            cout << help << endl;
            return 1;
      }
   if( !Y86::init ) exit( 6 );
   if( debug_level )
      cout << "Y86 simulator 64-bit, version " << version << endl;
   Y86 processor;
   if( 1 == argc - optind ) {                       // загрузка из командной строки   
      string iname( argv[ optind ] );
      if( string::npos == iname.find( '.' ) )
         iname += ".yo";
      ifstream fin;
      fin.open( iname );
      if( !fin ) {
         cerr << "ошибочное имя входного файла!" << endl;
         return 2;
      }
      fin >> processor;
      fin.close();
      if( !processor ) {
         cerr << "ошибочный код программы! : " << processor.errline() << endl;
         return 3;
      }
   }
   Y86 start = processor;
   while( true ) {   
      string e;
      cout << '.';
      getline( cin, e );
      if( cin.eof() ) {
         cout << endl; 
         return 0;
      }     
      if( !e.length() ) continue;
      const string mode( "cfrmsxdheq" );
      int icmd = mode.find( tolower( e[ 0 ] ) );
      if( e.length() > 1 ) {
         e = e.substr( 1 );
         string::size_type found = e.find_first_not_of( ' ' );
         if( found != string::npos ) e = e.substr( found );
      }
      switch( icmd >= 0 ? mode[ icmd ] : '~' ) {
         case 'c' :                                 // текущая строка кода
            cout << processor.current_line() << endl; 
            break;
         case 'f' :                                 // состояние процессора
            cout << processor.get_state() << endl; 
            break;
         case 'r' :                                 // регистры процессора
            cout << processor.get_regs(); 
            break;
         case 'm' :                                 // дамп памяти
            cout << processor.dump(); 
            break;
         case 'd' : 
            cout << processor.compare( start );     // изменения
            break;
         case 's' :                                 // выполнить шаг программы
            processor.step();
            break;
         case 'x' :                                 // ... последовательность команд
            if( !e.empty() ) 
               try { processor.execute( stoi( e ) ); }
               catch(...) { processor.execute(); }
            else      
               processor.execute();
            break;
         case 'h' :
            cout << "допустимые команды:";
            for( char c : mode ) cout << ' ' << c;
            cout << endl;
            break;            
         case 'e' :                                 // завершение интерпретатора 
         case 'q' :
            return 0;
         default :
            cout << "недопустимая команда" << endl; 
      }
   }
}
